function plotH=plotCross(pD,proj,colors)
%2D plot of projected Mean and StDev for an array of GaussD objects
%Input:
%pD=        a single GaussD object or multidim array of GaussD objects
%proj=      vector with indices of vector elements to plot
%           length(proj)==2 (but 3 should maybe also be allowed
%colors=    string with plot color codes, 'rgb' etc
%
%Result:
%plotH=    vector with plot handles to graph objects
%
%Arne Leijon 2006-03-14 tested
%           2011-05-26 minor cleanup
if nargin<2
    proj=[1 2];end;%default coords
if length(proj)~=2
    error('Can only plot 2-D');end;
if nargin<3
    colors='rbgk';end;
if isempty(colors)
    colors='rbgk';end;
nObj=numel(pD);
plotH=[];%storage for plot handles
for i=1:nObj
    cObj=colors(1+mod(i-1,length(colors)));%circulate among given colors
    m=pD(i).Mean;
    v=pD(i).CovEigen*diag(pD(i).StDev);%main distribution axes
    for k=1:size(v,2)%for each main axis
        xx=[(m-v(:,k)),(m+v(:,k))];%line start,end points
        xx=xx(proj,:);%selected coords for plot
        plotH=[plotH,plot(xx(1,:)',xx(2,:)',['-',cObj],'LineWidth',2)];
        hold on;
    end;
end;
hold off;
