%function [X,S]=rand(h,nSamples);
%HMM class method,
%generates a random sequence of data from a given Hidden Markov Model.
%
%Input:
%h= HMM object
%nSamples= maximum no of output samples (scalars or column vectors)
%
%Result:
%X= matrix or row vector with output data samples
%S= row vector with corresponding integer state values
%   obtained from the StateGen component.
%   nS= length(S) == size(X,2)= number of output samples.
%   If the StateGen can generate infinite-duration sequences,
%       nS == nSamples
%   If the StateGen is a finite-duration MarkovChain,
%       nS <= nSamples
%
%Arne Leijon 2004-11-24 tested
%           2006-04-18 included sub-state sequence U
%           2009-07-20 removed sub-state sequence again
%           2009-07-20 minor change for Matlab R2008a
%           2009-08-16 @MarkovChain/rand does NOT return END state 

function [X,S]=rand(h,nSamples)
if numel(h)>1
    error('Method works only for a single object');end;
S=rand(h.StateGen,nSamples);%the MarkovChain sub-object generates random state sequence
nX=length(S);
% if S(end)== h.StateGen.nStates+1% reached END state
%     nX=length(S)-1;%***should not be needed any more
% else
%     nX=length(S);
% end
dSize=h.DataSize;
X=zeros(dSize,nX);
for s=1:max(S)%each generated state index
    X(:,S==s)=rand(h.OutputDistr(s),sum(S==s));
end;