% Gustav Eje Henter 2012-11-01

classdef GaussKern % < ProbDistr
    properties (Access=public)
        Taus = 1;
        Isotropic = true;
        binRatio = 1;
    end
    properties (Dependent,Access=public)
        DataSize;%  size of data vectors
        Mean;%      total mean
        Variance;
    end
    methods (Access=public)
        function k = GaussKern(varargin)
            if (nargin < 1),
                return; % Default GaussKern
            elseif (nargin == 1),
                arg1 = varargin{1};
                if isa(arg1,'GaussKern')
                    k = arg1;
                    return;
                end
                k.Taus = arg1;
            end
        end
        
        % Initialization and training methods
        k = init(k,xs,ws);
        
        % Usage methods
        [lpzs,logS] = logprob(k,x1,x2,taus);
        [pzs,logS] = prob(k,x1,x2,taus);
        xs = rand(k,nSamples,taus);
        
        % Conditional methods
        [lpzs,logS] = marginalLogprob(k,x1,x2,givenDims,taus);
        xs = conditionalRand(k,nSamples,givenDims,taus);
        
        % Currently unimplemented
        %aState = adaptStart(k);
        %aState = adaptAccum(k,aState,obsData);
        %k = adaptSet(k,aState);
    end
    methods% get/set methods
        function D = get.DataSize(k)
            D = size(k.Taus,1);
        end
        function M = get.Mean(k)
            M = zeros(size(k.Taus));
        end
        function V = get.Variance(k)
            V = 1./k.Taus;
        end
    end
end