% Gustav Eje Henter 2013-04-04

function aState = adaptAccum(mm,aState,x,gamma)

nObj = numel(mm);

if (nargin < 4) || isempty(gamma)
    if mm(1).Crossval,
        gamma = prob(mm,[]);
    else
        gamma = prob(mm,xs);
    end
    gamma = gamma./repmat(sum(gamma,1),nObj,1);
end

for i = 1:nObj,
    aState(i) = accumSingle(mm(i),aState(i),x,gamma(i,:));
end

function aState = accumSingle(mm,aState,x,gamma)

o = mm.Order;
Dx = mm.DSize;
T = size(x,2);

aState.xs = [aState.xs x];
aState.lxs(end+1) = T;
aState.gammas{end+1} = gamma;

[lgrhos,ldrhos,pts] = getrhospts(mm,x);

Nx = size(pts,2);
Np = size(mm.Points,2);

% Regular rhos (normalized joint probability, including p+1)
grhos = exp(lgrhos);

% Discriminative responsibilities (without p+1)
drhos = exp(ldrhos);
%aState.ldrhos{end+1} = ldrhos;

aState.sumGamma = aState.sumGamma + sum(gamma);
aState.sumGammaRho = aState.sumGammaRho + exp(ldrhos)*gamma';

% Compute reverse-Jensen weights and sufficient statistics
taus = mm.Kern.Taus;

[omegas,omegap] = deal(zeros(Np,Nx));
sufstat = zeros(Dx*(o+1),1);
for i = 1:Dx*(o+1),
    d2nt = sqdistance(mm.Points(i,:),pts(i,:),1);
    
    if (i <= Dx*o)
        sufstat(i) = gamma*sum((grhos - drhos).*d2nt,1)';
        
        expectdev = taus(i)*d2nt - 1;
        omegap = max(omegap,expectdev);
        omegas = omegas + (expectdev).^2;
    else
        sufstat(i) = gamma*sum(grhos.*d2nt,1)';
    end
end

if mm.FastApproxUpdate,
    omegas = drhos.*(omegas + omegap); % Fast approximation
else
    omegas = 2*gfun(drhos./2).*omegas + drhos.*omegap; % Guaranteed ascent
end

aState.sumOmega = aState.sumOmega + gamma*sum(omegas,1)';

regweight = gamma*sum(drhos + omegas,1)';

aState.sumSufStat = aState.sumSufStat + sufstat;
aState.sumSufStat(1:Dx*o) = aState.sumSufStat(1:Dx*o) + regweight./taus(1:Dx*o);

function [lgrhos,ldrhos,pts] = getrhospts(mm,x)

Np = size(mm.Points,2);
o = mm.Order;
Dx = mm.DSize;

% Compute responsibilities
if mm.Crossval,
    [~,lgrhos] = mm.logprob([]);
    
    pts = mm.Points;
else
    [~,lgrhos] = mm.logprob(x);
    
    pts = mm.series2points(x,false);
end

% Compute discriminative component occupancies
lpzsmarg = mm.Kern.marginalLogprob(mm.Points,pts,[false(o*Dx,1);true(Dx,1)]);
ldrhos = lgrhos - lpzsmarg; % Cancel the p+1 term

ld = logsumexp(ldrhos,1);
ldrhos = ldrhos - repmat(ld,Np,1); % Normalize by dividing by the sum

function g = gfun(gam)

g = zeros(size(gam));

mask = (gam > 1/6);
g(mask) = gam(mask) + (1/(4*log(6)) + (5/(6*log(6)))^2 - 1/6);

mask = ~mask & (gam > 0);
g(mask) = ((gam(mask)-1)./log(gam(mask))).^2 - 1./(4*log(gam(mask)));