% Gustav Eje Henter 2012-11-07

classdef KDEMM < KernelDensity
    properties (Access=public)
        Order = 1;
        Stationary = true;
    end
    methods (Access=public)
        function mm = KDEMM(varargin)
            if (nargin < 1),
                return; % Default empty KDEMM
            elseif (nargin == 1),
                arg1 = varargin{1};
                if isa(arg1,'KDEMM')
                    mm = arg1;
                    return;
                elseif isa(arg1,'KernelDensity')
                    mm.Points = arg1.Points;
                    mm.Weights = arg1.Weights;
                    mm.Kern = arg1.Kern;
                    mm.Order = 1;
                    return;
                end
                ws = ones(1,size(arg1,2));
            else
                arg1 = varargin{1};
                ws = varargin{2};
            end
            mm.init(arg1,ws);
        end
        
        % Training methods
        mm = init(mm,xs,ws);
        %[mm,logprobs]=train(mm,xs,nIter,minStep);
        
        % Usage methods
        [xs,zs] = rand(mm,nSamples,bwparms);
        [p,rhos,logS] = prob(mm,xs,bwparms);
        [lp,rhos,logS] = logprob(mm,xs,bwparms);
        
        % Conditional methods
        [xs,zs] = conditionalRand(mm,x0,nSamples,bwparms);
        [p,rhos,logS] = conditionalProb(mm,xs,x0,bwparms);
        
        % Currently unimplemented
        function aState = adaptStart(mm)
            aState = [];
        end
        function aState = adaptAccum(mm,aState,obsData)
        end
        function mm = adaptSet(mm,aState)
        end
        
        % Get-set methods are purely inherited from KernelDensity
    end
end