% Gustav Eje Henter 2012-11-12

function [p,rhos,logS] = conditionalProb(mm,xs,x0,bwparms)

o = mm.Order;
N = mm.NumPoints;

if (nargin < 4),
    bwparms = [];
end

if (nargin < 2) || isempty(xs),
    xs = []; % Use self-log-prob with cross-validation, regardless of x0
elseif (nargin < 3) || isempty(x0),
    [p,rhos,logS] = prob(mm,xs,bwparms); % Assume the unconditional case
    return;
end

Nx = size(xs,2);
N0 = size(x0,2);
givenDims = 1:o;

if isempty(xs),
    % Use self-log-prob with cross-validation, regardless of x0
    [p,rhos,logS] = conditionalProb@KernelDensity(mm,[],givenDims,bwparms);
    
    if mm.Stationary,
        p = p(1:end-o);
        rhos = rhos(:,1:end-o);
        logS = logS(1:end-o);
    end
elseif (N0 >= o),
    xs = [x0(end-o+1:end) xs];
    
    xsegs = zeros(o+1,Nx);
    for p = 1:o+1,
        xsegs(p,:) = xs(p:(p+Nx-1));
    end
    
    [p,rhos,logS] = conditionalProb@KernelDensity(mm,xsegs,givenDims,bwparms);
else
    % Not enough data to compute a value
    p = [];
    rhos = zeros(N,0);
    logS = [];
    return;
end

if (nargout < 3),
    p = p.*exp(logS);
end