% Gustav Eje Henter 2012-11-28

function [p,logS,rhos] = singleProb(mm,xs,bwparms)

o = mm.Order;
N = mm.NumPoints;

if (nargin < 2) || isempty(xs),
    % Use self-log-prob with cross-validation
    crossval = true;
    Nx = mm.NumPoints;
    if (o <= 0),
        [p,logS,rhos] = singleProb@KernelDensity(mm,[],bwparms);
    else
        [p,logS,rhos] = conditionalProb(mm,[],[],bwparms);
    end
else
    crossval = false;
    Nx = size(xs,2);
    if (o <= 0),
        [p,logS,rhos] = singleProb@KernelDensity(mm,xs,bwparms);
    elseif (Nx > o),
        [p,logS,rhos] = conditionalProb(mm,xs(o+1:end),xs(1:o),bwparms);
    else
        % Not enough data to compute a value
        p = [];
        rhos = zeros(N,0);
        logS = [];
    end
end

% If stationary, include the marginal log-probability of the starting point
if mm.Stationary && (o > 0),
    if ~crossval,
        Nv = min(1+o,Nx);
        [p(1),logS(1),rhos(:,1)] = marginalProb(mm,xs(1:Nv)',1:Nv,bwparms);
    else
        [lpz0,logS(1)] = mm.Kern.logprob(mm.Points,mm.Points(:,1),bwparms);
        
        % Use cross-validation weights
        wm = mm.Weights';
        wm = wm./(1 - wm(1));
        wm(1) = 0;
        
        lpz0 = lpz0 + log(wm);
        
        % Recompute logS
        logS2 = max(lpz0,[],1);
        lpz0 = lpz0 - repmat(logS2,N,1);
        logS(1) = logS(1) + logS2;
        
        if (nargout > 1),
            rhos(:,1) = exp(lpz0)./sum(exp(lpz0),1);
        end
        
        p(1) = sum(exp(lpz0),1);
    end
end