% Gustav Eje Henter 2013-02-06

function [xs,zs] = conditionalRand(pD,x0s,givenDims)

if (numel(pD) ~= 1),
    error('This function only works for a single object!');
end

if (nargin < 2) || isempty(x0s),
    [xs,zs] = rand(pD,[]); % Assume the unconditional case
    return;
end

nSamples = size(x0s,2);

if (nargin < 3) || isempty(givenDims),
    [xs,zs] = rand(pD,nSamples); % Assume the unconditional case
    return;
end

D = size(pD.Points,1);

givenDims = pD.dims2logical(givenDims);

prunedin = (size(x0s,1) == sum(givenDims)); % Is input pruned?

xs = zeros(D,nSamples);
if prunedin,
    xs(givenDims,:) = x0s;
else
    xs(givenDims,:) = x0s(givenDims,:);
end

zs = zeros(1,nSamples);

[~,lpzconds] = singleMarginalLogprob(pD,xs,givenDims);
cmf = cumsum(exp(lpzconds),1);

for t = 1:nSamples,
    zs(t) = find(rand < cmf(:,t),1,'first');
end

xs(~givenDims,:) = pD.Points(~givenDims,zs);

%xs = xs + pD.Kern.rand(nSamples).*repmat(~givenDims,1,nSamples);
xs(~givenDims,:) = xs(~givenDims,:)...
    + pD.Kern.conditionalRand(nSamples,givenDims);

if prunedin,
    xs = xs(~givenDims,:); % Prune output as well
end