% Gustav Eje Henter 2013-02-01

function [pD,logprobs] = train(pD,maxIter)

if (numel(pD) ~= 1),
    error('This function only works for a single object!');
end

[D N] = size(pD.Points);

logprobs(1) = sum(pD.logprob)/(D*N);

if pD.Kern.Isotropic || (D == 1),
    % One-dimensional optimization
    opts = optimset(@fminsearch);
    opts.MaxIter = maxIter;
    opts.MaxFunEvals = [num2str(max(200,maxIter)) '*numberofvariables'];
    
    [logtau,lpopt,exitflag] =...
        fminsearch(@(lt) (-logprobOfLogTau(pD,lt)),log(pD.Kern.Taus(1)),opts);
else
    % Multidimensional optimization
    opts = optimset(@fminunc);
    opts.MaxIter = maxIter;
    opts.MaxFunEvals = [num2str(max(100,maxIter)) '*numberofvariables'];
    opts.Display = 'off';
    
    [logtau,lpopt,exitflag] =...
        fminunc(@(lt) (-logprobOfLogTau(pD,lt)),log(pD.Kern.Taus),opts);
end

pD.Kern.Taus = exp(logtau).*ones(D,1);

logprobs(2) = sum(pD.logprob)/(D*N);

function lp = logprobOfLogTau(pD,logtau)

[D N] = size(pD.Points);

pD.Kern.Taus = exp(logtau).*ones(D,1);

lp = sum(pD.logprob)/(D*N);