% Gustav Eje Henter 2013-02-16

function [pD,logprobs] = train(pD,maxIter)

if (numel(pD) ~= 1),
    error('This function only works for a single object!');
end

[D N] = size(pD.Points);

logprobs(1) = sum(pD.logprob)/(D*N);

tausused = unique(pD.Kern.Tied);

if (numel(tausused) == 1) || (D == 1),
    % One-dimensional optimization
    opts = optimset(@fminsearch);
    opts.MaxIter = maxIter;
    opts.MaxFunEvals = [num2str(max(200,maxIter)) '*numberofvariables'];
    
    [logtau,lpopt,exitflag] = fminsearch(@(lt) (-logprobOfLogTau(pD,lt)),...
        log(pD.Kern.Taus(tausused)),opts);
    
    pD.Kern.Taus(tausused) = exp(logtau);
else
    % Multidimensional optimization
    opts = optimset(@fminunc);
    opts.MaxIter = maxIter;
    opts.MaxFunEvals = [num2str(max(100,maxIter)) '*numberofvariables'];
    opts.Display = 'off';
    opts.LargeScale = 'off';
    
    [logtau,lpopt,exitflag] = fminunc(@(lt) (-logprobOfLogTau(pD,lt)),...
        log(pD.Kern.Taus),opts);
    
    pD.Kern.Taus = exp(logtau);
end

logprobs(2) = sum(pD.logprob)/(D*N);

function lp = logprobOfLogTau(pD,logtau)

[D N] = size(pD.Points);

pD.Kern.Taus = exp(logtau);

lp = sum(pD.logprob)/(D*N);