% Gustav Eje Henter 2013-05-03

function [hmm,hmm0,lps] = MakeErgodicGaussHMM(initvar,xs,miniter,minstep)

if (nargin < 3) || isempty(miniter),
    miniter = 10;
end

if (nargin < 4) || isempty(minstep),
    minstep = log(1.01);
end

if isa(initvar,'HMM'),
    mc = initvar.StateGen;
    gammas = initvar.stateOccupancy(xs);
elseif isa(initvar,'GaussMixD'),
    [mc,gammas] = initFromGaussMixD(initvar,xs);
elseif isa(initvar,'GaussD'),
    gm = GaussMixD(initvar);
    [mc,gammas] = initFromGaussMixD(gm,xs);
elseif isa(initvar,'double'),
    if isempty(initvar)
        [hmm,hmm0,lps] = makeSingleStateHMM(xs);
        return;
    elseif (numel(initvar) == 1),
        nStates = initvar;
        [~,gm] = gmm_assign(nStates,0,xs,miniter,minstep);
        [mc,gammas] = initFromGaussMixD(gm,xs);
    else
        gammas = initvar;
        mc = gammas2mc(gammas);
    end
else
    error(['Unrecognized initialization class "' class(initvar) '."']);
end

nStates = size(gammas,1);

if (nStates <= 1),
    [hmm,hmm0,lps] = makeSingleStateHMM(xs);
    return;
end

gs = repmat(GaussD,nStates,1);
for s = 1:nStates,
    gs(s) = gamma2gauss(xs,gammas(s,:));
end


hmm0 = HMM('StateGen',mc,'OutputDistr',gs);

if (nStates <= 1), % EM converges instantly if there is only one state
    miniter = 1;
    minstep = Inf;
end

[hmm,lps] = hmm0.train(xs,size(xs,2),miniter,minstep);
% disp(lps)

%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [hmm,hmm0,lps] = makeSingleStateHMM(xs)

gd = GaussD('Mean',mean(xs,2),'StDev',std(xs,1,2));
mc = MarkovChain('TransitionProb',1);

[hmm,hmm0] = deal(HMM('StateGen',mc,'OutputDistr',gd));
lps = sum(gd.logprob(xs));

function [mc,rhos] = initFromGaussMixD(gd,xs)

%nS = numel(gd.MixWeight);
[Dx Nx] = size(xs);
Dg = gd.DataSize;
go = Dg/Dx-1;

pts = make_pts(xs,go);

rhos = gmm_rhos(gd,pts);
mc   = gammas2mc(rhos);

function gd = gamma2gauss(xs,gammas)

nX = size(xs,2);
nG = size(gammas,2);

if (nX > nG),
    xs = xs(:,end-nG+1:end);
    nX = nG;
else
    gammas = gammas(:,end-nX+1:end);
end

mu = xs*gammas';
res = xs - repmat(mu,1,nX);
sigma = (res.*res)*gammas';

gd = GaussD('Mean',mu,'StDev',sigma);