% Gustav Eje Henter 2020-05-10

% A quick demonstration of how to compute and graph conditional
% probabilitiy densities for trained models

clear;

%new_laser_3k_phaseinit; % Need only be run once, or use the original .mat
% load('./laser_3k_phaseinit/laser_3k_phaseinit_results.mat');

% Set up a fine grid for plotting
xrange = [0,256];
xres = 8*256+1; % Total number of points to evaluate for plot

xpts = diff(xrange)*((0:(xres-1))/(xres-1)) + xrange(1);
xstep = diff(xpts(1:2));

ncond = 9;
cond = xs(1:ncond); % Condition on the first points of the training data

% Uncomment this to plot the time series:
%figure(1);
%plot([cond,xs(ncond+1)]);
%hold on; plot(ncond+1,xs(ncond+1),'r*'); hold off;

% Which model to compute the conditional pdf for (examples):
%model = amt(3,1); % AR process of order 2
model = mmpts(3,1); % Stationary KDE-MM of order 2
%model = hmmt(end); % The biggest trained HMM we have
%model = kdeht(3,end); % The biggest trained KDE-HMM we have of order 2

clps = condlogprob(model,cond,xpts'); % Compute conditional pdf at xpts

figure(1);
plot(xpts,clps);

title(['Conditional density of x(' int2str(ncond+1)...
    ') given xs(1:' int2str(ncond) ')']);
xlabel(['x(' int2str(ncond+1) ')']);
ylabel('Conditional log-pdf ln(p)');

saveas(gcf,'condlogprob_demo_logpdf','fig');

curraxis = axis;
axis([xrange curraxis(3:4)]);

condprobs = exp(clps);

figure(2);
plot(xpts,condprobs);

title(['Conditional density of x(' int2str(ncond+1)...
    ') given xs(1:' int2str(ncond) ')']);
xlabel(['x(' int2str(ncond+1) ')']);
ylabel('Conditional pdf');

saveas(gcf,'condlogprob_demo_pdf','fig');

curraxis = axis;
axis([xrange curraxis(3:4)]);

% Numerical integration of the conditional probability distribution
% This should approach 1 in the limit of dense datapoints on the real line
totalcondprob = sum(condprobs)*xstep,
