% Gustav Eje Henter 2013-06-23
% only stationary kDE-MMs
% file: kdehmm_experiment5.m (12-07-2020)
% run file: run_SET100_ALL2n.m   with: 
%
% default_opts3;  ntrain = 500; ntest = 300;
% maxorderm = 5;  % for AR(p) and KDE-MM
% maxorderh = 3;  % p in AR-HMM and KDE-HMM
% maxstates = 5;  % states ns for G-HMM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function kdehmm_experiment5(datastr,ntrain,ntest,gridres,condq,...
    maxorderm,maxorderh,...
    maxstates,maxiter,nreinits,inititer,miniter,minstep,initarg,...
    initarg2,xv,xt)

starttime = now;

disp(['KDE-HMM experiment started on dataset "' datastr '" at '...
    datestr(starttime,31) '...']);

savefigs = false;
set(0,'DefaultFigureVisible','off');

outpath   = ['.' filesep datastr filesep];  % filesep = \
matsuffix = '_results';

matfile = [outpath datastr matsuffix];

xs    = xt;
x2s   = [xs(:,1:end-1);xs(:,2:end)];

%%%%%%% Train KDEs %%%%%%%%%%

kde10           = KernelDensity(xs);
[kde1t,lpskde1] = kde10.train(maxiter);

kde20           = KernelDensity(x2s);
[kde2t,lpskde2] = kde20.train(maxiter);

% Set up conditional probabilities (cut through 2D KDE)
givenDims = 1;

%%%%% Estimate Gaussian parameters %%%%%%%%%%

mux    = mean(xs);
sigmax = std(xs,1);

lpnormt = mean(log(normpdf(xs,mux,sigmax)));
lpnormv = mean(log(normpdf(xv,mux,sigmax)));

%%%%%%%%% Train AR models  %%%%%%%%%%
disp('AR models');

[am0,amt]     = deal(repmat(ARHMM,maxorderm+1,1));
lpsamt        = cell(maxorderm+1,1);
[lpsamt{:}]   = deal(-Inf);
[lpamt,lpamv] = deal(zeros(maxorderm+1,1));

lpamt(1) = lpnormt;
lpamv(1) = lpnormv;

for o = 1:maxorderm,
    [amt(o+1),am0(o+1),lpsamt{o+1}] = MakeErgodicARHMM(1,o,xs,inititer,minstep);
    
    lpamt(o+1)  = amt(o+1).logprob(xs)/(ntrain-o);
    lpamv(o+1)  = amt(o+1).logprob(xv)/(ntest-o);
    BIC_AR(o+1) = -2*lpamt(o+1)+o*log(ntrain);   % training
    
    disp(['p=' int2str(o) ', ns=' int2str(1)...
        ': lpt =' num2str(lpamt(o+1)) ', lpv =' num2str(lpamv(o+1))...
        ', BIC =' num2str(BIC_AR(o+1))]); % training
end

[~,bestamo_t] = nanmax(lpamt); % training
bestam_t      = amt(bestamo_t);
bestamo_t     = bestamo_t - 1;

[~,bestamo_v] = nanmax(lpamv); % validation
bestam_v      = amt(bestamo_v);
bestamo_v     = bestamo_v - 1;

[~,index_AR_BIC] = nanmax(BIC_AR);
best_AR_BIC      = index_AR_BIC-1;

%%%%% Train stationary and nonstationary KDE-MMs  %%%%%%%%%%
disp('KDE-MMs');

[mmpts,mmp0s] = deal(repmat(KDEMM,maxorderm+1,2));

lpsmmt        = cell(maxorderm+1,2);
[lpmmt,lpmmv] = deal(zeros(maxorderm+1,2));

statstrs = {'stationary','nonstationary'}; % stationary = 0
for nonstat = 0:0,
    for o = 0:maxorderm,
        mmp0s(o+1,1+nonstat).Order = o;
        mmp0s(o+1,1+nonstat).Stationary = (nonstat == 0);
        
        mmp0s(o+1,1+nonstat) = mmp0s(o+1,1+nonstat).init(xs);
        
        [mmpts(o+1,1+nonstat),lpsmmt{o+1,1+nonstat}]...
            = mmp0s(o+1,1+nonstat).train(maxiter);
        
        lpmmt(o+1,1+nonstat)  = sum(mmpts(o+1,1+nonstat).logprob(xs))/(ntrain-o);
        lpmmv(o+1,1+nonstat)  = sum(mmpts(o+1,1+nonstat).logprob(xv))/(ntest-o);
        BIC_KDE_MM(o+1,1+nonstat) = -2*lpmmt(o+1,1+nonstat)+o*log(ntrain);
 
        disp(['p =' int2str(o) ', ' statstrs{1+nonstat}...
            ': lpt =' num2str(lpmmt(o+1,1+nonstat))...
            ', lpv =' num2str(lpmmv(o+1,1+nonstat))...
            ', BIC =' num2str(BIC_KDE_MM(o+1,1+nonstat)) ]);
    end
end

[~,bestmmos_t_mm] = nanmax(lpmmt(:,1)); % stationary
bestmmos_t_mm     = bestmmos_t_mm - 1;

[~,bestmmos_v_mm] = nanmax(lpmmv(:,1));
bestmmos_v_mm     = bestmmos_v_mm - 1;

[~,index_KDE_MM_BIC] = nanmax(BIC_KDE_MM);
best_KDE_MM_BIC      = index_KDE_MM_BIC-1;

%% [~,bestmmos] = nanmax(lpmmv(:,1)); 
%% bestmmos     = bestmmos - 1;

%% [~,bestmmon] = nanmax(lpmmv(:,2)); % non stationary
%% bestmmon     = bestmmon - 1;

save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')

% Train initialization distributions (k-means or GMMs)

nss = 1:maxstates;

if (numel(initarg) < 2),
    % Use general-purpose initialization (k-means++ or GMMs)
    initorder    = initarg2;
    hmmgammainit = false;
    
    initgammas = cell(maxstates,nreinits);
    if initarg,    % GMM initialization
        initds     = repmat(GaussMixD,maxstates,nreinits);
        initgmmlps = cell(maxstates,nreinits);
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),initgmmlps{ns,r}]...
                    = gmm_assign(ns,initorder,xs,inititer,minstep,true);
            end
        end
    else % k-means++ initialization
        initds   = repmat(VQ,maxstates,nreinits);
        vqsigmas = cell(maxstates,nreinits);
        %initvqsigma = 0;  % Use hard assignment (bad for KDE-HMMs)
        initvqsigma  = -1; % Soft assignment with ML-estimated standard deviation
        for ns = nss,
            for r = 1:nreinits,
                [initgammas{ns,r},initds(ns,r),vqsigmas{ns,r}]...
                    = vq_assign(ns,initorder,xs,true,initvqsigma);
            end
        end
    end
else
    initgammas   = initarg; % Initial gamma values have already been supplied
    hmmgammainit = initarg2; % Whether or not to initialize from (AR)-HMMs
end

%%%%%%%%% Train Gaussian HMMs %%%%%%%%%%
disp('Gaussian HMMs');

[hmmt,hmm0,lpht,lphv,besthmm,bestho,besthns,lpsht,lpshr,lpdminh,...
    ttimeh,bestrh,BIC_G_HMM,best_G_HMM_BIC] ...
    = train_block_G_HMM(HMM,@createHMM,false,0,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);

save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')
%%%%%%%%% Train AR-HMMs %%%%%%%%%%
disp('AR-HMMs');

[art,ar0,lpart,lparv,bestar,bestaro,bestarns,lpsart,lpsarr,lpdminar,...
    ttimear,bestrar,BIC_AR_HMM,best_AR_HMM_BIC]...
    = train_block_AR_HMM(ARHMM,@createARHMM,...
    false,1:maxorderh,nss,...
    xs,initgammas,inititer,miniter,minstep,xv,matfile);

bstates = bestar.nStates;
bo      = bestar.OutputDistr(1).Order;

gammas = initgammas{bstates,bestrar(bo,bstates)}; % Assumes nss = 1:maxstates
gammas = gammas(:,1:end-bo);
[~,qs] = nanmax(gammas,[],1);

save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')

bstates = bestar.nStates;
bo      = bestar.OutputDistr(1).Order;

gammas = bestar.stateOccupancy(xs);
[~,qs] = nanmax(gammas,[],1);

save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')

%%%%%%%% Train KDE-HMMs %%%%%%%%%%
disp('KDE-HMMs');

if hmmgammainit,
    % Use alignments from best (AR-)HMMs to initialize KDE-HMMs
    kdehinitgammas = cell(maxstates,1);
    for ns = nss,
        [bestarlpatns,bestaroatns] = nanmax(lpart(:,ns)); % Ignore NaNs
        if (lpht(1,ns) >= bestarlpatns),
            kdehinitgammas{ns} = hmmt(1,ns).stateOccupancy(xs);
        else
            kdehinitgammas{ns} = art(bestaroatns,ns).stateOccupancy(xs);
        end
    end
else
    kdehinitgammas = initgammas;
end

% disp(bestarns)           % best # of states from AR-HMM
% disp(bestaro)            % best AR(p) order from AR-HMM

[index_s_AR_HMM, index_p_AR_HMM]= maxmat(BIC_AR_HMM); 
% index_s_AR_HMM = AR order p; number of states
% disp(index_s)
% disp(index_p)
 
% disp (nss)
%% nss = numel(nss);       % change
%% maxorderh = bestarns;
initorder = index_s_AR_HMM; %  best AR order from AR-HMM
nss       = index_p_AR_HMM; %  best # of states from AR-HMM

% return

% for one maxorderh (AR order p) remove 0: in 3rd line. That is 0:maxorderh

[kdeht,kdeh0,lpkdeht,lpkdehv,bestkdeh,bestkdeho,bestkdehns,...
 lpskdeht,lpskdehr,lpdminkdeh,ttimekdeh,bestarkdeh,BIC_KDE_HMM,best_KDE_HMM_BIC]...
 = train_block_KDE_HMM(KDEHMM,@createKDEHMM,true,maxorderh,nss,...  
 xs,kdehinitgammas,inititer,miniter,minstep,xv,matfile);

gammas  = bestkdeh.stateOccupancy(xs);
xkdehrt = bestkdeh.conditionalRand(xs(:,1:bestkdeho),ntrain-bestkdeho,...
    gammas(:,1)); % Start from a reasonable state, given xs context

%%%%%%% Save results and exit %%%%%%%%%%

endtime = now;

save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')

disp(['KDE-HMM experiment successfully completed at ' datestr(endtime,31) '.']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%  functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%% Gaussian HMMs  %%%%%%%%%%%%%%%%%%%%
function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
  ttime,bestr,BIC_G_HMM,best_G_HMM_BIC] ...
  = train_block_G_HMM(hmmexample,createfun,trainempty,os,nss,...
  xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);  % number of array elements
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt]   = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr]   = deal(cell(no,nnss));
[lpst{:}]     = deal(-Inf);
[lpsr{:}]     = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]     = deal(-Inf*ones(no,nnss));
lpdmin        = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

 for o = 1:no       % 1:1
    for ns = 1:nnss  % 1:2
        totaliter = 0;
        begintime = clock;
        begincpu  = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp   = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns)  = hmmttmp;
                hmm0(o,ns)  = hmm0tmp;
                lpst{o,ns}  = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp   = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns)       = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns)       = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        BIC_G_HMM(o,ns) = -2*lpt(o,ns)+os(o)*log(ntrain);
       
        disp(['p =' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt =' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', BIC =' num2str(BIC_G_HMM(o,ns))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration

        save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      
%       Save partial training results
    end  % end ns = 1:nnss 
end      % end o=1:no 

[~,besti]      = nanmax(lpv(:));
besthmm        = hmmt(besti);
[besto,bestns] = ind2sub(size(lpv),besti);

besto  = os(besto);
bestns = nss(bestns);

[~,best_G_HMM_BIC] = nanmax(BIC_G_HMM); % maximum value
save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [hmm,hmm0,lps] = createHMM(ns,o,initvar,xs,inititer,minstep)

[hmm,hmm0,lps] = MakeErgodicGaussHMM(initvar,xs,inititer,minstep);

function [ar,ar0,lps] = createARHMM(ns,o,initvar,xs,inititer,minstep)

[ar,ar0,lps] = MakeErgodicARHMM(initvar,o,xs,inititer,minstep);

function [kdeh,kdeh0,lps] = createKDEHMM(ns,o,initvar,xs,inititer,minstep)

[kdeh,kdeh0,lps] = MakeErgodicKDEHMM(initvar,o,xs,inititer,minstep);

%%%%%%%%%%%%%%%%%%%%%% AR-HMMs %%%%%%%%%%%%%%%%%%%%
function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
 ttime,bestr,BIC_AR_HMM,best_AR_HMM_BIC] ...
  = train_block_AR_HMM(hmmexample,createfun,trainempty,os,nss,...
  xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);  % number of array elements
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt]   = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr]   = deal(cell(no,nnss));
[lpst{:}]     = deal(-Inf);
[lpsr{:}]     = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]     = deal(-Inf*ones(no,nnss));
lpdmin        = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

 for o = 1:no,
    for ns = 1:nnss    % 1:2
        totaliter = 0;
        begintime = clock;
        begincpu  = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp   = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns)  = hmmttmp;
                hmm0(o,ns)  = hmm0tmp;
                lpst{o,ns}  = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp   = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu  = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns)        = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns)        = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        BIC_AR_HMM(o,ns) = -2*lpt(o,ns)+os(o)*log(ntrain);
       
        disp(['p =' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt =' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', BIC =' num2str(BIC_AR_HMM(o,ns))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration

        save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      
%       Save partial training results
    end  % end ns = 1:nnss 
end      % end o=1:no 

[~,besti]      = nanmax(lpv(:));
besthmm        = hmmt(besti);
[besto,bestns] = ind2sub(size(lpv),besti);

[~,best_AR_HMM_BIC] = nanmax(BIC_AR_HMM); % maximum value

besto  = os(besto);
% bestns = nss(bestns_AR_HMM);
save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      

%%%%%%%%%%%%%% KDE-HMMs with fixed value of p  %%%%%%%%%%%%%%%%%%%%
function [hmmt,hmm0,lpt,lpv,besthmm,besto,bestns,lpst,lpsr,lpdmin,...
 ttime,bestr,BIC_KDE_HMM,best_KDE_HMM_BIC] = train_block_KDE_HMM(hmmexample,createfun,trainempty,os,nss,...
    xs,initvars,inititer,miniter,minstep,xv,matfile)

ntrain = size(xs,2);
ntest  = size(xv,2);

no   = numel(os);  % number of array elements
nnss = numel(nss);

nreinits = size(initvars,2);

[hmm0,hmmt]   = deal(repmat(hmmexample,no,nnss));
[lpst,lpsr]   = deal(cell(no,nnss));
[lpst{:}]     = deal(-Inf);
[lpsr{:}]     = deal(repmat({-Inf},1,nreinits));
[lpt,lpv]     = deal(-Inf*ones(no,nnss));
lpdmin        = Inf*ones(no,nnss);
[ttime,bestr] = deal(zeros(no,nnss));

 for o = 1:no,
    for ns = 1:nnss   % 1:1
        totaliter = 0;
        begintime = clock;
        begincpu  = cputime;
        for r = 1:nreinits, % Train many initial models
            [hmmttmp,hmm0tmp,lpsr{o,ns}{r}]...
                = createfun(nss(ns),os(o),initvars{ns,r},xs,inititer,Inf);
            
            totaliter = totaliter + numel(lpsr{o,ns}{r});
            dmintmp   = nanmin(diff(lpsr{o,ns}{r}));
            if ~isempty(dmintmp),
                lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
            end
            % Keep track of the most promising model
            if (lpsr{o,ns}{r}(end) > lpst{o,ns}(end)),
                bestr(o,ns) = r;
                hmmt(o,ns)  = hmmttmp;
                hmm0(o,ns)  = hmm0tmp;
                lpst{o,ns}  = lpsr{o,ns}{r};
            end
        end
        
        % Perform additional training on the most promising candidate
        if trainempty,
            [hmmt(o,ns),lpstmp] = hmmt(o,ns).train([],[],miniter,minstep);
        else
            [hmmt(o,ns),lpstmp]...
                = hmmt(o,ns).train(xs,ntrain,miniter,minstep);
        end
        lpst{o,ns} = [lpst{o,ns} lpstmp];
        
        totaliter = totaliter + numel(lpstmp);
        dmintmp   = nanmin(diff(lpst{o,ns}));
        if ~isempty(dmintmp),
            lpdmin(o,ns) = min(lpdmin(o,ns),dmintmp);
        end
        endtime = clock; % Record time required
        endcpu = cputime;
        
        % Reset initial distribution, since we assume stationarity
        if (ns > 1),% && ~any(any(isnan(hmmt(o,ns).StateGen.TransitionProb))),
            try
                hmmt(o,ns).StateGen = hmmt(o,ns).StateGen.setStationary;
            catch err
                warning('kdehmm_experiment:setStationary',...
                    ['Could not set stationary distribution due to error "'...
                    err.message '"']);
            end
        end
        
        % Evaluate performance of stationarized models
        lpt(o,ns)         = hmmt(o,ns).logprob(xs)/(ntrain-os(o));
        lpv(o,ns)         = hmmt(o,ns).logprob(xv)/(ntest-os(o));
        BIC_KDE_HMM(o,ns) = -2*lpt(o,ns)+os(o)*log(ntrain);
       
        disp(['p =' int2str(os(o)) ', ns=' int2str(nss(ns))...
            ': lpt =' num2str(lpt(o,ns)) ', lpv=' num2str(lpv(o,ns))...
            ', BIC =' num2str(BIC_KDE_HMM(o,ns))]);
        
        ttime(o,ns) = (endcpu - begincpu)/totaliter; % Store time per iteration

        save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      
%       Save partial training results
    end  % end ns = 1:nnss 
end      % end o=1:no 

[~,besti]      = nanmax(lpv(:));
besthmm        = hmmt(besti);
[besto,bestns] = ind2sub(size(lpv),besti);

[~,best_KDE_HMM_BIC] = nanmax(BIC_KDE_HMM); % maximum value

besto  = os(besto);
bestns = nss(bestns);
save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')      



 