% Gustav Eje Henter 2013-05-10

function [rhos,thevq,sigmas] = vq_assign(ns,order,xs,prepad,sigmas)

if (nargin < 1) || isempty(ns),
    ns = 1;
end

if (nargin < 2) || isempty(order),
    order = 0;
end

if (nargin < 4) || isempty(prepad),
    prepad = false;
end

if (nargin < 5) || isempty(sigmas),
    sigmas = 0; % Use hard assignment
end

[Dx,Nx] = size(xs);
[pts,Dp,Np] = make_pts(xs,order);

if (ns == 1),
    thevq = VQ;
    thevq.CodeBook = mean(pts,2);
    
    rhos = ones(1,Np);
    return;
end

recreate = true;
while recreate,
    thevq = create(VQ,pts,ns);
    recreate = (size(thevq.CodeBook,2) ~= ns);
end

indices = thevq.encode(pts);

if any(sigmas < 0), % Estimate tied sigmas
    res = pts - thevq.CodeBook(:,indices);
    sigmas = sqrt(mean(res.*res,2));
end

ntile = Dp/numel(sigmas);
sigmas = repmat(reshape(sigmas,[],1),ntile,1);

if any(sigmas == 0), % Hard assignment
    rhos = double(repmat((1:ns)',1,Np) == repmat(indices,ns,1));
    
    if prepad,
        rhos = [zeros(ns,order) rhos];
        shortvq = thevq;
        for o = 1:order,
            shortvq.CodeBook = thevq.CodeBook(1:Dx*o,:);
            tempind = shortvq.encode(pts(1:Dx*o,1));
            rhos(tempind,o) = 1;
        end
    end
else % Soft assignment based on analogous GMM
    gs = repmat(GaussD,ns,1);
    for s = 1:ns,
        gs(s) = GaussD('Mean',thevq.CodeBook(:,s),'StDev',sigmas);
    end
    
    gmd = GaussMixD(gs);
    rhos = gmm_rhos(gmd,pts,Dx*prepad);
end